@@ -1,6 +1,7 @@
 import sys
 import sysconfig
 import os
+import logging
 import re
 import errno
 import shlex
@@ -46,6 +47,8 @@ from ..gui.utils import message_warning, message_information, \
                         OSX_NSURL_toLocalFile
 from ..help.manager import get_dist_meta, trim, parse_meta
 
+log = logging.getLogger(__name__)
+
 OFFICIAL_ADDONS = [
     "Orange-Bioinformatics",
     "Orange3-DataFusion",
@@ -492,13 +495,14 @@ class AddonManagerDialog(QDialog):
 
         try:
             packages = f.result()
-        except (IOError, OSError) as err:
+        except (IOError, OSError, ValueError) as err:
             message_warning(
                 "Could not retrieve package list",
                 title="Error",
                 informative_text=str(err),
                 parent=self
             )
+            log.error(str(err), exc_info=True)
             packages = []
         except Exception:
             raise
@@ -584,7 +588,7 @@ class AddonManagerDialog(QDialog):
             if path.endswith(self.ADDON_EXTENSIONS):
                 name, vers, summary, descr = (get_meta_from_archive(path) or
                                               (os.path.basename(path), '', '', ''))
-                names.append(name)
+                names.append(cleanup(name))
                 packages.append(
                     Installable(name, vers, summary,
                                 descr or summary, path, [path]))
