@@ -160,6 +160,9 @@ class SchemeEditWidget(QWidget):
         self.__widgetMenu.addAction(self.__removeSelectedAction)
         self.__widgetMenu.addSeparator()
         self.__widgetMenu.addAction(self.__helpAction)
+        if log.isEnabledFor(logging.DEBUG):
+            self.__widgetMenu.addSeparator()
+            self.__widgetMenu.addAction(self.__showSettingsAction)
 
         self.__linkMenu = QMenu(self.tr("Link"), self)
         self.__linkMenu.addAction(self.__linkEnableAction)
@@ -285,6 +288,13 @@ class SchemeEditWidget(QWidget):
                     enabled=False
                     )
 
+        self.__showSettingsAction = \
+            QAction(self.tr("Show settings"), self,
+                    objectName="show-settings",
+                    toolTip=self.tr("Show widget settings"),
+                    triggered=self.showSettings,
+                    enabled=False)
+
         shortcuts = [Qt.Key_Delete,
                      Qt.ControlModifier + Qt.Key_Backspace]
 
@@ -844,6 +854,15 @@ class SchemeEditWidget(QWidget):
                 )
         self.__undoStack.endMacro()
 
+    def showSettings(self):
+        """
+        Dump settings of selected items to the standard output.
+        """
+        selected = self.scene().selectedItems()
+        for item in selected:
+            node = self.scene().node_for_item(item)
+            self.scheme().dump_settings(node)
+
     def selectAll(self):
         """
         Select all selectable items in the scheme.
@@ -1264,6 +1283,7 @@ class SchemeEditWidget(QWidget):
         self.__helpAction.setEnabled(len(nodes) == 1)
         self.__renameAction.setEnabled(len(nodes) == 1)
         self.__duplicateSelectedAction.setEnabled(bool(nodes))
+        self.__showSettingsAction.setEnabled(len(nodes) == 1)
 
         if len(nodes) > 1:
             self.__openSelectedAction.setText(self.tr("Open All"))
