@@ -591,17 +591,18 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         text = self.tiptexts.get(int(modifiers), self.tiptexts[0])
         self.tip_textitem.setHtml(text)
 
-    def new_data(self, data, subset_data=None, **args):
-        self.plot_widget.clear()
-        self.remove_legend()
+    def new_data(self, data, subset_data=None, new=True, **args):
+        if new:
+            self.plot_widget.clear()
+            self.remove_legend()
 
-        self.density_img = None
-        self.scatterplot_item = None
-        self.scatterplot_item_sel = None
-        self.reg_line_item = None
-        self.labels = []
-        self.selection = None
-        self.valid_data = None
+            self.density_img = None
+            self.scatterplot_item = None
+            self.scatterplot_item_sel = None
+            self.reg_line_item = None
+            self.labels = []
+            self.selection = None
+            self.valid_data = None
 
         self.subset_indices = set(e.id for e in subset_data) if subset_data else None
 
@@ -632,6 +633,12 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.master.Information.missing_coords.clear()
         self._clear_plot_widget()
 
+        if self.shown_y != attr_y:
+            # 'reset' the axis text width estimation. Without this the left
+            # axis tick labels space only ever expands
+            yaxis = self.plot_widget.getAxis("left")
+            yaxis.textWidth = 30
+
         self.shown_x, self.shown_y = attr_x, attr_y
 
         if self.jittered_data is None or not len(self.jittered_data):
@@ -776,13 +783,15 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         return size_data
 
     def update_sizes(self):
+        self.master.prepare_data()
+        self.update_point_size()
+
+    def update_point_size(self):
         if self.scatterplot_item:
             size_data = self.compute_sizes()
             self.scatterplot_item.setSize(size_data)
             self.scatterplot_item_sel.setSize(size_data + SELECTION_WIDTH)
 
-    update_point_size = update_sizes
-
     def get_color_index(self):
         if self.attr_color is None:
             return -1
@@ -906,6 +915,10 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         return pen, brush
 
     def update_colors(self, keep_colors=False):
+        self.master.update_colors()
+        self.update_alpha_value(keep_colors)
+
+    def update_alpha_value(self, keep_colors=False):
         if self.scatterplot_item:
             pen_data, brush_data = self.compute_colors(keep_colors)
             pen_data_sel, brush_data_sel = self.compute_colors_sel(keep_colors)
@@ -921,8 +934,6 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                 elif self.density_img:
                     self.plot_widget.removeItem(self.density_img)
 
-    update_alpha_value = update_colors
-
     def create_labels(self):
         for x, y in zip(*self.scatterplot_item.getData()):
             ti = TextItem()
@@ -930,15 +941,21 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             ti.setPos(x, y)
             self.labels.append(ti)
 
+    def _create_label_column(self):
+        if self.attr_label in self.data.domain:
+            return self.data.get_column_view(self.attr_label)[0]
+        return self.master.data.get_column_view(self.attr_label)[0]
+
     def update_labels(self):
         if self.attr_label is None or \
                 self.label_only_selected and self.selection is None:
             for label in self.labels:
                 label.setText("")
             return
+        self.assure_attribute_present(self.attr_label)
         if not self.labels:
             self.create_labels()
-        label_column = self.data.get_column_view(self.attr_label)[0]
+        label_column = self._create_label_column()
         formatter = self.attr_label.str_val
         label_data = map(formatter, label_column)
         black = pg.mkColor(0, 0, 0)
@@ -971,11 +988,16 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         return shape_data
 
     def update_shapes(self):
+        self.assure_attribute_present(self.attr_shape)
         if self.scatterplot_item:
             shape_data = self.compute_symbols()
             self.scatterplot_item.setSymbol(shape_data)
         self.make_legend()
 
+    def assure_attribute_present(self, attr):
+        if self.data is not None and attr not in self.data.domain:
+            self.master.prepare_data()
+
     def update_grid(self):
         self.plot_widget.showGrid(x=self.show_grid, y=self.show_grid)
 
@@ -1042,12 +1064,11 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if not self.legend:
             self.create_legend()
         shape_var = self.domain[shape_index]
-        color = self.plot_widget.palette().color(OWPalette.Data)
-        pen = QPen(color.darker(self.DarkerValue))
+        color = QColor(0, 0, 0)
         color.setAlpha(self.alpha_value)
         for i, value in enumerate(shape_var.values):
             self.legend.addItem(
-                ScatterPlotItem(pen=pen, brush=color, size=10,
+                ScatterPlotItem(pen=color, brush=color, size=10,
                                 symbol=self.CurveSymbols[i]), escape(value))
 
     def zoom_button_clicked(self):
